(* ::Package:: *)

(* AutoDipole - Package of an automated dipole subtraction by
Kouhei Hasegawa, Sven Moch, and Peter Uwer, 2009 *)


(* Filename:virtual.m  :creation of I-term *)


(* [Code to pdoduce all I-terms (massless and massive cases)]
Outline
1. Identify inputted process
2. Create I-terms 
3. Functions to be called  *)


(* Input *)


Itermcreation[ilist_,flist_]:=Module[{},identifyV[ilist,flist];createV;]


(* 1. Identify inputted process *)


identifyV[ilist_,flist_]:=Module[{},assignV[ilist,flist];masslistV;processV;dicountsetV;plistV]


assignV[ilist_,flist_]:=Module[{},ilistmV=ilist;flistmV=flist;XV[1]=Part[ilist,1];
              XV[2]=Part[ilist,2];nfinalv=Length[flist];nfield=2+nfinalv;Do[XV[i+2]=Part[flist,i],{i,1,nfinalv}]]


masslistV:=Module[{},mlist={};Do[
                If[XV[i]==g||XV[i]==u||XV[i]==ubar||XV[i]==d||XV[i]==dbar,m[i]=0];
                If[XV[i]==b||XV[i]==bbar,m[i]=mb];
                If[XV[i]==t||XV[i]==tbar,m[i]=mt];
\[NonBreakingSpace]               If[XV[i]==Wp||XV[i]==Wm,m[i]=mw];
\[NonBreakingSpace]               If[XV[i]==Z,m[i]=mz];
                If[XV[i]==e||XV[i]==ebar,m[i]=0];
                If[XV[i]==muon||XV[i]==muonbar,m[i]=0];
\[NonBreakingSpace]               If[XV[i]==h,m[i]=mh];
                AppendTo[mlist,m[i]];
                ,{i,3,nfield}];
                Print["Masses: {0,0} --> ", mlist];massesV={mb,mt};m[1]=0;m[2]=0;]


processV:=Module[{},tfinalV=Table[{XV[i],p[i]},{i,3,nfield}];
                 Print["Virtual correction: ",{{XV[1],p[1]},{XV[2],p[2]}}," --> ", tfinalV];]


quarksV={u,ubar,d,dbar,b,bbar,t,tbar};


othersV={e,ebar,muon,muonbar,gamma,Wp,Wm,Z,h};


masses={mb,mt};


lenmF=Length[mFlist];


plistV:=Module[{},glistV={};ulistV={};ubarlistV={};dlistV={};dbarlistV={};
                          blistV={};bbarlistV={};tlistV={};tbarlistV={};
                 Do[If[XV[i]==g,AppendTo[glistV,i],Null],{i,3,nfield}];
                 Do[If[XV[i]==u,AppendTo[ulistV,i]],{i,3,nfield}];
                 Do[If[XV[i]==ubar,AppendTo[ubarlistV,i]],{i,3,nfield}];
                 Do[If[XV[i]==d,AppendTo[dlistV,i]],{i,3,nfield}];
                 Do[If[XV[i]==dbar,AppendTo[dbarlistV,i]],{i,3,nfield}];
                 Do[If[XV[i]==b,AppendTo[blistV,i]],{i,3,nfield}];
                 Do[If[XV[i]==bbar,AppendTo[bbarlistV,i]],{i,3,nfield}];
                 Do[If[XV[i]==t,AppendTo[tlistV,i]],{i,3,nfield}];
                 Do[If[XV[i]==tbar,AppendTo[tbarlistV,i]],{i,3,nfield}];
                 ]


dicountsetV:=Module[{},coV=0;Do[dcV[n]=0;dMcV[n]=0;dsV[n]=0;dssV[n]=0;dMssV[n]=0;,{n,1,144}];]


(* 2. Create I-terms and print outputs *)


createV:=Module[{},ItermA;ItermB;ItermC;ItermD;diprintV;interfaceCLBS;]


ItermA:=Module[{},Print["{(1):i=f}"];
Print["[1.(i,k)=(f,k): I1(i,k)]"];Iterm1;
Print["[2.(i,b)=(f,b): I2(i,b)]"];Iterm2;]


ItermB:=Module[{},Print["{(2):i=g}"];
Print["[3.(i,k)=(g,k): I3(i,k)]"];Iterm3;
Print["[4.(i,b)=(g,b): I4(i,b)]"];Iterm4;]


ItermC:=Module[{},Print["{(3):a=f}"];
Print["[5.(a,k)=(f,k): I5(a,k)]"];Iterm5;
Print["[6.(a,b)=(f,b): I6(a,b)"];Iterm6;]


ItermD:=Module[{},Print["{(4):a=g}"];
Print["[7.(a,k)=(g,k): I7(a,k)]"];Iterm7;
Print["[8.(a,b)=(g,b): I8(a,b)]"];Iterm8;]


(* 1.(i,k)=(f,k): I1 (i,k) *)


Iterm1:=Do[If[MemberQ[quarksV,XV[i]],
                 Do[If[k != i && FreeQ[othersV,XV[k]],
         coV=coV+1;
Print[" "];
Print["--I(",coV,")--"];
   If[m[i]==0 && m[k]==0,
         dcV[1]=dcV[1]+1;
         Iterm[coV]=Insf[i,k];
         If[sijexpansion==1,Iterm[coV]=Insfun[i,k]];
         Print["Ifk(",i,k,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[i,k];
      ];
   If[MemberQ[massesV,m[i]] || MemberQ[massesV,m[k]],
         dMcV[1]=dMcV[1]+1;
         Iterm[coV]=IM1[i,k];
         If[sijexpansion==1,Iterm[coV]=IM1un[i,k]];
         Print["IMfk(",i,k,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[i,k];
      ];
                    ],{k,3,nfield}]
                  ],{i,3,nfield}]


(* 2.(i,b)=(f,b): I2 (i,b) *)


Iterm2:=Do[If[MemberQ[quarksV,XV[i]],
                 Do[If[ FreeQ[othersV,XV[b]],
         coV=coV+1;
Print[" "];
Print["--I(",coV,")--"];
   If[m[i]==0,
         dcV[2]=dcV[2]+1;
         Iterm[coV]=Insf[i,b];
         If[sijexpansion==1,Iterm[coV]=Insfun[i,b]];
         Print["Ifb(",i,b,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[i,b];
      ];
   If[MemberQ[massesV,m[i]],
         dMcV[2]=dMcV[2]+1;
         Iterm[coV]=IM2[i,b];
         If[sijexpansion==1,Iterm[coV]=IM2un[i,b]];
         Print["IMfb(",i,b,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[i,b];
      ];
                    ],{b,1,2}]
                  ],{i,3,nfield}]


(* 3.(i,k)=(g,k): I3 (i,k) *)


Iterm3:=Do[If[ XV[i]==g,
                 Do[If[k!=i && FreeQ[othersV,XV[k]],
         coV=coV+1;
Print[" "];
Print["--I(",coV,")--"];
   If[m[k]==0 && lenmF==0,
         dcV[3]=dcV[3]+1;
         Iterm[coV]=Insg[i,k];
         If[sijexpansion==1,Iterm[coV]=Insgun[i,k]];
         Print["Igk(",i,k,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[i,k];
               ];
   If[MemberQ[massesV,m[k]] || lenmF != 0,
         dMcV[3]=dMcV[3]+1;
         Iterm[coV]=IM3[i,k];
         If[sijexpansion==1,Iterm[coV]=IM3un[i,k]];
         Print["IMgk(",i,k,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[i,k];
      ];
                    ],{k,3,nfield}]
                  ],{i,3,nfield}]


(* 4.(i,b)=(g,b): I4 (i,b) *)


Iterm4:=Do[If[ XV[i]==g,
                 Do[If[FreeQ[othersV,XV[b]],
         coV=coV+1;
Print[" "];
Print["--I(",coV,")--"];
   If[lenmF==0,
         dcV[4]=dcV[4]+1;
         Iterm[coV]=Insg[i,b];
         If[sijexpansion==1,Iterm[coV]=Insgun[i,b]];
         Print["Igb(",i,b,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[i,b];
      ];
   If[lenmF != 0,
         dMcV[4]=dMcV[4]+1;
         Iterm[coV]=IM4[i,b];
         If[sijexpansion==1,Iterm[coV]=IM4un[i,b]];
         Print["IMgb(",i,b,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[i,b];
      ];
                    ],{b,1,2}]
                  ],{i,3,nfield}]


(* 5.(a,k)=(f,k): I5 (a,k) *)


Iterm5:=Do[If[MemberQ[quarksV,XV[a]],
                 Do[If[FreeQ[othersV,XV[k]],
         coV=coV+1;
Print[" "];
Print["--I(",coV,")--"];
   If[m[k]==0,
         dcV[5]=dcV[5]+1;
         Iterm[coV]=Insf[a,k];
         If[sijexpansion==1,Iterm[coV]=Insfun[a,k]];
         Print["Ifk(",a,k,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[a,k];
      ];
   If[MemberQ[massesV,m[k]],
         dMcV[5]=dMcV[5]+1;
         Iterm[coV]=IM5[a,k];
         If[sijexpansion==1,Iterm[coV]=IM5un[a,k]];
         Print["IMfk(",a,k,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[a,k];
      ];
                    ],{k,3,nfield}]
                  ],{a,1,2}]


(* 6.(a,b)=(f,b): I6 (a,b) *)


Iterm6:=Do[If[MemberQ[quarksV,XV[a]],
                 Do[If[ b != a && FreeQ[othersV,XV[b]],
         coV=coV+1;
Print[" "];
Print["--I(",coV,")--"];
         dcV[6]=dcV[6]+1;
         Iterm[coV]=Insf[a,b];
         If[sijexpansion==1,Iterm[coV]=Insfun[a,b]];
         Print["Ifb(",a,b,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[a,b];
                    ],{b,1,2}]
                 ],{a,1,2}]


(* 7.(a,k)=(g,k): I7 (a,k) *)


Iterm7:=Do[If[XV[a]==g,
                 Do[If[FreeQ[othersV,XV[k]],
         coV=coV+1;
Print[" "];
Print["--I(",coV,")--"];
   If[m[k]==0,
         dcV[7]=dcV[7]+1;
         Iterm[coV]=Insg[a,k];
         If[sijexpansion==1,Iterm[coV]=Insgun[a,k]];
         Print["Igk(",a,k,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[a,k];
      ];
   If[MemberQ[massesV,m[k]],
         dMcV[7]=dMcV[7]+1;
         Iterm[coV]=IM7[a,k];
         If[sijexpansion==1,Iterm[coV]=IM7un[a,k]];
         Print["IMgk(",a,k,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[a,k];
      ];
                    ],{k,3,nfield}]
                  ],{a,1,2}]


(* 8.(a,b)=(g,b): I8 (a,b) *)


Iterm8:=Do[If[XV[a]==g,
                 Do[If[ b != a && FreeQ[othersV,XV[b]],
         coV=coV+1;
Print[" "];
Print["--I(",coV,")--"];
         dcV[8]=dcV[8]+1;
         Iterm[coV]=Insg[a,b];
         If[sijexpansion==1,Iterm[coV]=Insgun[a,b]];
         Print["Igb(",a,b,")=",Iterm[coV]];
         constructexp[coV];
         PrintCLV[a,b];
                    ],{b,1,2}]
                 ],{a,1,2}]


diprintV:=Module[{},
Do[dsV[n]=dcV[n]+dMcV[n];,{n,1,8}];
dssV[1]=dsV[1]+dsV[2];dssV[2]=dsV[3]+dsV[4];dssV[3]=dsV[5]+dsV[6];dssV[4]=dsV[7]+dsV[8];
dMssV[1]=dMcV[1]+dMcV[2];dMssV[2]=dMcV[3]+dMcV[4];dMssV[3]=dMcV[5]+dMcV[6];dMssV[4]=dMcV[7]+dMcV[8];
itotal=dssV[1]+dssV[2]+dssV[3]+dssV[4];
iMtotal=dMssV[1]+dMssV[2]+dMssV[3]+dMssV[4];
Print["--------------------------------------"];
Print["Number of I terms"];
Print["{(1):i=f}:  ", dssV[1], " (", dMssV[1],")"];
Print["              [1.(i,k)=(f,k): I1(i,k)]   ", dsV[1], " (", dMcV[1],")"];
Print["              [2.(i,b)=(f,b): I2(i,b)]   ", dsV[2], " (", dMcV[2],")"];
Print["{(2):i=g}:  ", dssV[2]," (", dMssV[2],")"];
Print["              [3.(i,k)=(g,k): I3(i,k)]   ", dsV[3], " (", dMcV[3],")"];
Print["              [4.(i,b)=(g,b): I4(i,b)]   ", dsV[4], " (", dMcV[4],")"];
Print["{(3):a=f}:  ", dssV[3]," (", dMssV[3],")"];
Print["              [5.(a,k)=(f,k): I5(a,k)]   ", dsV[5], " (", dMcV[5],")"];
Print["              [6.(a,b)=(f,b): I6(a,b)]   ", dsV[6], " (", dMcV[6],")"];
Print["{(4):a=g}:  ", dssV[4]," (", dMssV[4],")"];
Print["              [7.(a,k)=(g,k): I7(a,k)]]   ", dsV[7], " (", dMcV[7],")"];
Print["              [8.(a,b)=(g,b): I8(a,b)]   ", dsV[8], " (", dMcV[8],")"];
Print["-------------------"];
 (* Total*)
If[itotal==coV,Print["[Total] : ",itotal];,Print["Count error"];];
Print[" (Massive I terms : ",iMtotal,")"];
Print["-------------------"];
Print["END"];
]


interfaceCLBS:=Module[{},
Do[matchkind[i]=-1;matchV[i]=0;listV[i]=0;lenV[i]=0;listV2[i]=0;lenV2[i]=0;
exmomV[i]={};exmomV2[i]={};,{i,1,itotal}];
Do[Do[exlistV[i,k]={};mapmom[i,k]=0,{k,1,20}],{i,1,itotal}];
Do[Do[Do[exlistV2[i,k,l]={0,0},{l,1,20}],{k,1,20}],{i,1,itotal}];

Do[
  (*Print["\n Iterm(",i,") starts\n"];*)
  Do[If[emitV[i]==emit[j] && specV[i]==spec[j],matchkind[i]=0;matchV[i]=j;Break[]];,{j,1,dtotal}];
  If[matchV[i]==0,
    If[emitV[i] <= 2 && specV[i] <= 2, matchkind[i]=-2;Print["CLV(",i,") does not match in real part"]];
    If[emitV[i] >= 3 && specV[i] <= 2, emitspecficase[i]];
    If[emitV[i] <= 2 && specV[i] >= 3, emitspecifcase[i]];
    If[emitV[i] >= 3 && specV[i] >= 3, emitspecffcase[i]];
   ];
,{i,1,itotal}];

]


emitspecficase[i_]:=Module[{},
  Print["Iterm(",i,"): (",emitV[i],",",specV[i],")=(f,i)"];
  calllistV[emitV[i]];
  Do[exlistV[i,k]={Part[listV[i],k],specV[i]},{k,1,lenV[i]}];
   Do[
    Do[
      If[Part[exlistV[i,k],1] == emit[j] && specV[i] == spec[j], matchV[i]=j;
      exmomV[i]={emitV[i],emit[j]};matchkind[i]=1;Break[]] 
    ,{j,1,dtotal}];
      If[matchV[i] != 0, Break[]]
   ,{k,1,lenV[i]}]; 
];


emitspecifcase[i_]:=Module[{},
   Print["Iterm(",i,"): (",emitV[i],",",specV[i],")=(i,f)"];
   calllistV[specV[i]];
   Do[exlistV[i,k]={emitV[i],Part[listV[i],k]},{k,1,lenV[i]}];
    Do[
     Do[
       If[emitV[i] == emit[j] && Part[exlistV[i,k],2] == spec[j], matchV[i]=j;
        exmomV[i]={spec[j],specV[i]};matchkind[i]=1;Break[]] 
      ,{j,1,dtotal}];
     If[matchV[i] != 0, Break[]]
  ,{k,1,lenV[i]}]; 
];


emitspecffcase[i_]:=Module[{},
    Print["Iterm(",i,"): (",emitV[i],",",specV[i],")=(f,f)"];
     calllistV[emitV[i]];calllistV2[specV[i]];
 Do[
   Do[
      If[Part[listV[i],k] != Part[listV2[i],l],exlistV2[i,k,l]={Part[listV[i],k],Part[listV2[i],l]};]
    ,{k,1,lenV[i]}]
  ,{l,1,lenV2[i]}];

 Do[
   Do[   
     Do[  
      If[Part[exlistV2[i,k,l],1] == emit[j] && Part[exlistV2[i,k,l],2] == spec[j], matchV[i]=j;
          exmomV[i]={emit[j],emitV[i]};exmomV2[i]={spec[j],specV[i]};
          reductioncase[i];
          If[matchkind[i]==-1,nonreductioncase[i];];
          Break[]
        ];
     ,{j,1,dtotal}];
   ,{l,1,lenV2[i]}];
   If[matchV[i] != 0, Break[]]
 ,{k,1,lenV[i]}];

];


reductioncase[i_]:=Module[{},
    Print["Reduction case"];
       If[ Part[exmomV[i],1]== Part[exmomV[i],2],matchkind[i]=1;exmomV[i]=exmomV2[i];exmomV2[i]={};Break[]];
       If[ Part[exmomV2[i],1]== Part[exmomV2[i],2],matchkind[i]=1;exmomV2[i]={};Break[]];
       If[ Part[exmomV[i],1] == Part[exmomV2[i],2] && Part[exmomV[i],2] == Part[exmomV2[i],1],
         matchkind[i]=1;exmomV2[i]={};Break[]];
];


nonreductioncase[i_]:=Module[{},
    Print["Non-reduction case"];
 If[ Part[exmomV[i],2]== Part[exmomV2[i],1] && Part[exmomV[i],1]!= Part[exmomV2[i],2],matchkind[i]=2;
    Print["Triangle case 1 of Non-reduction case"];
   Do[
     If[ al != Part[exmomV[i],1] && al != Part[exmomV[i],2] && al != Part[exmomV2[i],2],mapmom[i,al]=al];
     If[ al == Part[exmomV[i],1], mapmom[i,al]=Part[exmomV[i],2]];
     If[ al == Part[exmomV2[i],1], mapmom[i,al]=Part[exmomV2[i],2]];
     If[ al == Part[exmomV2[i],2], mapmom[i,al]=Part[exmomV[i],1]];
     ,{al,3,nfield}];Break[]
   ];

 If[ Part[exmomV[i],1]== Part[exmomV2[i],2] && Part[exmomV[i],2]!= Part[exmomV2[i],1],matchkind[i]=2;
    Print["Triangle case 2 of Non-reduction case"];
        Do[
          If[ al != Part[exmomV[i],1] && al != Part[exmomV[i],2] && al != Part[exmomV2[i],1],
              mapmom[i,al]=al];
          If[ al == Part[exmomV[i],1], mapmom[i,al]=Part[exmomV[i],2]];
          If[ al == Part[exmomV2[i],1], mapmom[i,al]=Part[exmomV2[i],2]];
          If[ al == Part[exmomV[i],2], mapmom[i,al]=Part[exmomV2[i],1]];
        ,{al,3,nfield}];Break[]
      ];

 If[ Part[exmomV[i],1] != Part[exmomV2[i],2] && Part[exmomV[i],2]!= Part[exmomV2[i],1],matchkind[i]=2;
    Print["Double exchange case of Non-reduction case"];
       Do[
        If[ al != Part[exmomV[i],1] && al != Part[exmomV[i],2] && al != Part[exmomV2[i],1] 
             && al != Part[exmomV2[i],2], mapmom[i,al]=al];
        If[ al == Part[exmomV[i],1], mapmom[i,al]=Part[exmomV[i],2]];
        If[ al == Part[exmomV[i],2], mapmom[i,al]=Part[exmomV[i],1]];
        If[ al == Part[exmomV2[i],1], mapmom[i,al]=Part[exmomV2[i],2]];
        If[ al == Part[exmomV2[i],2], mapmom[i,al]=Part[exmomV2[i],1]];
     ,{al,3,nfield}];Break[] 
   ]; 
];


calllistV[leg_]:=Module[{},
temp=XV[leg];temp2=ToString[temp];temp3=StringJoin[temp2,"listV"];listV[i]=ToExpression[temp3];
lenV[i]=Length[listV[i]]]


calllistV2[leg_]:=Module[{},
temp=XV[leg];temp2=ToString[temp];temp3=StringJoin[temp2,"listV"];listV2[i]=ToExpression[temp3];
lenV2[i]=Length[listV2[i]]]


(* 3. Common functions to be called *)


(* (1) i=f (or fbar) *)


(* [1.(i,k)=(f,k)] *)


Insf[i_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CF^(-1)*Nuf*
(mu^(2)/sc[i,k])^(eps)//Simplify


Insfun[i_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CF^(-1)*Nuf*msos[i,k]//Simplify


   (* Massive case*)


IM1[i_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CF^(-1)*(CF*(mu^(2)/sc[i,k])^(eps)*
(NuM1[i,k]-Pi^2*3^(-1)) + CGamf[i] + gamf*Log[mu^(2)/sc[i,k]]+ gamf + Kf)//Simplify


IM1un[i_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CF^(-1)*(CF*msos[i,k]*
(NuM1[i,k]-Pi^2*3^(-1)) + CGamf[i] + gamf*Log[mu^(2)/sc[i,k]]+ gamf + Kf)//Simplify


NuM1[i_,k_]:=Which[
MatchQ[m[i],x_Symbol] && MatchQ[m[k],x_Symbol],NuM1mm[i,k],
MatchQ[m[i],x_Symbol] && m[k] == 0,NuM1mz[i,k],
m[i] == 0 && MatchQ[m[k],x_Symbol],NuM1zm[i,k]];


NuM1mm[i_,k_]:=NuMSmm[i,k] + NuMNSmmf[i,k]//Simplify


NuM1mz[i_,k_]:=NuMSm[i,k] + NuMNSmzf[i,k]//Simplify


NuM1zm[i_,k_]:=NuMSm[k,i] + NuMNSzmf[i,k]//Simplify


(* [2.(i,b)=(f,b)] *)


   (* Massive case*)


IM2[i_,b_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CF^(-1)*(CF*(mu^(2)/sc[i,b])^(eps)*
(NuM2[i,b]-Pi^2*3^(-1)) + CGamMf[i] + gamf*Log[mu^(2)/sc[i,b]]+ gamf + Kf)//Simplify


IM2un[i_,b_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CF^(-1)*(CF*msos[i,b]*
(NuM2[i,b]-Pi^2*3^(-1)) + CGamMf[i] + gamf*Log[mu^(2)/sc[i,b]]+ gamf + Kf)//Simplify


NuM2[i_,b_]:= NuMSm[i,b] + NuMNSmzf[i,b] //Simplify


(* (2) i=g *)


(* [3.(i,k)=(g,k)] *)


Insg[i_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CA^(-1)*Nug*
(mu^(2)/sc[i,k])^(eps)//Simplify


Insgun[i_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CA^(-1)*Nug*
msos[i,k]//Simplify


   (* Massive case*)


IM3[i_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CA^(-1)*(CA*(mu^(2)/sc[i,k])^(eps)*
(NuM3[i,k]-Pi^2*3^(-1)) + CGamg + gamg*Log[mu^(2)/sc[i,k]]+ gamg + Kg)//Simplify


IM3un[i_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CA^(-1)*(CA*msos[i,k]*
(NuM3[i,k]-Pi^2*3^(-1)) + CGamg + gamg*Log[mu^(2)/sc[i,k]]+ gamg + Kg)//Simplify


NuM3[i_,k_]:=Which[
MatchQ[m[k],x_Symbol] && lenmF != 0,NuM3mm[i,k],
MatchQ[m[k],x_Symbol] && lenmF == 0,NuM3mz[i,k],
m[k] == 0 && lenmF != 0,NuM3zm[i,k]];


NuM3mm[i_,k_]:=NuMSm[k,i] + NuMNSzmmg[i,k]//Simplify;


NuM3mz[i_,k_]:=NuMSm[k,i] + NuMNSzmzg[i,k]//Simplify;


NuM3zm[i_,k_]:=NuMSz + NuMNSzzmg[i,k]//Simplify;


(* [4.(i,b)=(g,b)] *)


   (* Massive case*)


IM4[i_,b_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CA^(-1)*(CA*(mu^(2)/sc[i,b])^(eps)*
(NuM4[i,b]-Pi^2*3^(-1)) + CGamMg + gamg*Log[mu^(2)/sc[i,b]]+ gamg + Kg)//Simplify


IM4un[i_,b_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CA^(-1)*(CA*msos[i,b]*
(NuM4[i,b]-Pi^2*3^(-1)) + CGamMg + gamg*Log[mu^(2)/sc[i,b]]+ gamg + Kg)//Simplify


NuM4[i_,b_]:= NuMSz + NuMNSzzmg[i,b] //Simplify


(* (3) a=f (or fbar) *)


(* [5.(a,k)=(f,k)] *)


   (* Massive case*)


IM5[a_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CF^(-1)*(CF*(mu^(2)/sc[a,k])^(eps)*
(NuM5[a,k]-Pi^2*3^(-1)) + CGam0f + gamf*Log[mu^(2)/sc[a,k]]+ gamf + Kf)//Simplify


IM5un[a_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CF^(-1)*(CF*msos[a,k]*
(NuM5[a,k]-Pi^2*3^(-1)) + CGam0f + gamf*Log[mu^(2)/sc[a,k]]+ gamf + Kf)//Simplify


NuM5[a_,k_]:= NuMSm[k,a] + NuMNSzmf[a,k] //Simplify


(* [6.(a,b)=(f,b)] *)


(* (4) a=g *)


(* [7.(a,k)=(g,k)] *)


   (* Massive case*)


IM7[a_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CA^(-1)*(CA*(mu^(2)/sc[a,k])^(eps)*
(NuM7[a,k]-Pi^2*3^(-1)) + CGam0g + gamg*Log[mu^(2)/sc[a,k]]+ gamg + Kg)//Simplify


IM7un[a_,k_]:= -AL*(2*Pi)^(-1)*((4*Pi)^(eps)/Gamma[1-eps])*CLV[coV]*CA^(-1)*(CA*msos[a,k]*
(NuM7[a,k]-Pi^2*3^(-1)) + CGam0g + gamg*Log[mu^(2)/sc[a,k]]+ gamg + Kg)//Simplify


NuM7[a_,k_]:= NuMSm[k,a] + NuMNSzmzg7[a,k] //Simplify


(* [8.(a,b)=(g,b)] *)


(*  Variables *)


(* For massless I-terms *)


Nuf:=Nufg;


Nug:=2^(-1)*Nugg + Nf*Nuffbar;


Nufg:=CF*(eps^(-2)+3*(2*eps)^(-1)+5-Pi^2*2^(-1));


Nugg:=2*CA*(eps^(-2)+11*(6*eps)^(-1)+50*9^(-1)-Pi^2*2^(-1));


Nuffbar:=TR*(-2*(3*eps)^(-1)-16*9^(-1));


gamf:=3*2^(-1)*CF;


gamg:=11*6^(-1)*CA - 2*3^(-1)*TR*Nf;


Kf:=CF*(7*2^(-1)-Pi^2*6^(-1));


Kg:=CA*(67*18^(-1)-Pi^2*6^(-1)) - 10*9^(-1)*TR*Nf;


(* For massive I-terms *)


(* NuMS *)


NuMSmm[i_,k_]:=v2[i,k]^(-1)*(
eps^(-1)*Log[rho[i,k]] - 4^(-1)*Log[(rhon[i,k,i])^2]^(2) - 4^(-1)*Log[(rhon[i,k,k])^2]^(2) - Pi^2*6^(-1)
) + v2[i,k]^(-1)*Log[rho[i,k]]*Log[Qiksq[i,k]/sc[i,k]];


NuMSm[i_,k_]:= (2*eps^2)^(-1) + (2*eps)^(-1)*Log[m[i]^2/sc[i,k]] - 4^(-1)*(Log[m[i]^2/sc[i,k]])^2 - Pi^2*12^(-1)- 
2^(-1)*Log[m[i]^2/sc[i,k]]*Log[sc[i,k]/Qiksq[i,k]] - 2^(-1)*Log[m[i]^2/Qiksq[i,k]]*Log[sc[i,k]/Qiksq[i,k]];


NuMSz:=eps^(-2);


(* NuMNSf *)


NuMNSmmf[i_,k_]:=gamf*CF^(-1)*Log[sc[i,k]/Qiksq[i,k]]+ v2[i,k]^(-1)*(Log[rho[i,k]^2]*Log[1+rho[i,k]^2] +
 2*Li2[rho[i,k]^2] - Li2[1-rhon[i,k,i]^2] - Li2[1-rhon[i,k,k]^2]-Pi^2*6^(-1))+Log[(Qik[i,k] - m[k])*Qik[i,k]^(-1)]-
2*Log[((Qik[i,k] - m[k])^2 - m[i]^2)*Qiksq[i,k]^(-1)]-2*m[i]^2*sc[i,k]^(-1)*Log[m[k]*(Qik[i,k] - m[k])^(-1)]-
m[k]*(Qik[i,k] - m[k])^(-1) +2*m[k]*(2*m[k] - Qik[i,k])*sc[i,k]^(-1)+Pi^2*2^(-1);


NuMNSmzf[i_,k_]:=gamf*CF^(-1)*Log[sc[i,k]/Qiksq[i,k]]+ Pi^2*6^(-1) - Li2[sc[i,k]/Qiksq[i,k]]-
2*Log[sc[i,k]/Qiksq[i,k]] - m[i]^2/sc[i,k]*Log[m[i]^2/Qiksq[i,k]];


NuMNSzmf[i_,k_]:=gamf*CF^(-1)*(
Log[sc[i,k]/Qiksq[i,k]] - 2*Log[(Qik[i,k] - m[k])*Qik[i,k]^(-1)] - 2*m[k]*(m[k]+Qik[i,k])^(-1))+
Pi^2*6^(-1)-Li2[sc[i,k]/Qiksq[i,k]];


(* NuMNSg *)


NuMNSzmmg[i_,k_]:=NuMNSzmzg[i,k]+2*3^(-1)*TR*CA^(-1)*Sum[Log[Part[mFlist,n]^2/Qaux^2],{n,1,lenmF}]+
4*3^(-1)*TR*CA^(-1)*Sum[dec[sc[i,k],m[k],Part[mFlist,n]]*nFik1[i,k,Part[mFlist,n]],{n,1,lenmF}]+
(kap2-2*3^(-1))*m[k]^2/sc[i,k]*2*TR*CA^(-1)*
Sum[dec[sc[i,k],m[k],Part[mFlist,n]]*nFik2[i,k,Part[mFlist,n]],{n,1,lenmF}];


NuMNSzmzg[i_,k_]:=gamg*CA^(-1)*(Log[sc[i,k]/Qiksq[i,k]] - 2*Log[(Qik[i,k] - m[k])*Qik[i,k]^(-1)] - 
2*m[k]*(m[k]+Qik[i,k])^(-1))+Pi^2*6^(-1)-Li2[sc[i,k]/Qiksq[i,k]]+
(kap2-2*3^(-1))*m[k]^2/sc[i,k]*(2*TR*CA^(-1)*Nf-1)*Log[2*m[k]/(Qik[i,k]+m[k])];


NuMNSzmzg7[i_,k_]:=gamg*CA^(-1)*(Log[sc[i,k]/Qiksq[i,k]] - 2*Log[(Qik[i,k] - m[k])*Qik[i,k]^(-1)] - 
2*m[k]*(m[k]+Qik[i,k])^(-1))+Pi^2*6^(-1)-Li2[sc[i,k]/Qiksq[i,k]];


NuMNSzzmg[i_,k_]:=2*3^(-1)*TR*CA^(-1)*Sum[Log[Part[mFlist,n]^2/Qaux^2],{n,1,lenmF}]+
4*3^(-1)*TR*CA^(-1)*Sum[dec[sc[i,k],m[k],Part[mFlist,n]]*nFik3[i,k,Part[mFlist,n]],{n,1,lenmF}];


nFik1[i_,k_,mF_]:=Log[(Qik[i,k]-m[k])*Qik[i,k]^(-1)]+m[k]*rho1[i,k,mF]^3*(Qik[i,k]+m[k])^(-1)+
Log[(1+rho1[i,k,mF])*2^(-1)]-rho1[i,k,mF]*3^(-1)*(3+rho1[i,k,mF]^2)-2^(-1)*Log[mF^2*Qiksq[i,k]^(-1)];


nFik2[i_,k_,mF_]:=rho2[i,k,mF]^2*Log[(rho2[i,k,mF]-rho1[i,k,mF])*(rho2[i,k,mF]+rho1[i,k,mF])^(-1)]-
Log[(1-rho1[i,k,mF])*(1+rho1[i,k,mF])^(-1)] - 8*rho1[i,k,mF]*mF^2*sc[i,k]^(-1);


nFik3[i_,k_,mF_]:=Log[(1+rho1[i,k,mF])*2^(-1)]-rho1[i,k,mF]*3^(-1)*(3+rho1[i,k,mF]^2)-2^(-1)*Log[mF^2*
sc[i,k]^(-1)];


(* Gammaf and Gammag *)


CGamf[i_]:= Which[MatchQ[m[i],x_Symbol],CGamMf[i],m[i] == 0,CGam0f];


CGam0f:=eps^(-1)*gamf;


CGamMf[i_]:=CF*(eps^(-1)+2^(-1)*Log[m[i]^2*mu^(-2)] - 2);


CGamg:=Which[lenmF == 0,CGam0g,lenmF != 0,CGamMg];


CGam0g:=eps^(-1)*gamg;


CGamMg:=CGam0g - 2*3^(-1)*TR*Sum[Log[Part[mFlist,n]^2/Qaux^2],{n,1,lenmF}];


(* rho *)


rho[i_,k_]:=Sqrt[(1-v2[i,k])*(1+v2[i,k])^(-1)]//Simplify;


rhon[i_,k_,n_]:=Sqrt[(1-v2[i,k]+2*munsq[i,k,n]/(1-munsq[i,k,i]-munsq[i,k,k]))*
(1+v2[i,k]+2*munsq[i,k,n]/(1-munsq[i,k,i]-munsq[i,k,k]))^(-1)]//Simplify;


rho1[i_,k_,mF_]:=Sqrt[1-4*mF^2*(Qik[i,k]-m[k])^(-2)];


rho2[i_,k_,mF_]:=Sqrt[1-4*mF^2*(Qiksq[i,k]-m[k]^2)^(-1)];


v2[i_,k_]:=(sc[i,k])^(-1)*Sqrt[(sc[i,k])^(2)-(4*m[i]^2*m[k]^2)];


munsq[i_,k_,n_]:=mun[i,k,n]^2;


mun[i_,k_,n_]:=m[n]*Qik[i,k]^(-1);


Qik[i_,k_]:=Sqrt[Qiksq[i,k]]


Qiksq[i_,k_]:=sc[i,k] + m[i]^2 + m[k]^2;


sc[i_,j_]:=If[i>j,tmp={ToString[s],ToString[j],ToString[i]};
tmp2=StringJoin[tmp];ToExpression[tmp2],
tmp={ToString[s],ToString[i],ToString[j]};
tmp2=StringJoin[tmp];ToExpression[tmp2]];


msos[i_,k_]:=If[i < k,mos[i,k],mos[k,i]];


constructexp[coV_]:=Module[{},
         If[ replistVirtual != {},
         Iterm[coV]=Iterm[coV]/.replistVirtual;
         ];
         Itermexp[coV]=Series[Iterm[coV],{eps,0,0}];
         Itermexp[coV]=Itermexp[coV]/.{EulerGamma -> Eul};
         Itermcoef[coV,-2]=Coefficient[Itermexp[coV],eps^(-2)]//Simplify;
         Itermcoef[coV,-1]=Coefficient[Itermexp[coV],eps^(-1)]//Simplify;
         Itermcoef[coV,0]=Coefficient[Itermexp[coV],eps,0]//Simplify;
         Print["  I(",coV,",-2)=",Itermcoef[coV,-2]];
         Print["  I(",coV,",-1)=",Itermcoef[coV,-1]];
         Print["  I(",coV,",0)=",Itermcoef[coV,0]];
];


PrintCLV[i_,k_]:=Module[{},Print["CLV(",coV,")=CLBS(",i,",",k,")"];
emitV[coV]=i;specV[coV]=k;];
